package xyz.tbvns.ui.LevelEditor.tabs;

import javafx.scene.Node;
import javafx.scene.control.*;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.Pane;
import javafx.stage.FileChooser;
import xyz.tbvns.core.Decoder;
import xyz.tbvns.core.Objects.Level;
import xyz.tbvns.ui.LevelEditor.FXTab;
import xyz.tbvns.core.UIProgressBar;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;

public class EditLevelString implements FXTab {
    private FlowPane pane;
    private TextArea area;
    private ToolBar toolBar;

    @Override
    public Pane getPane(Level level) {
        pane = new FlowPane();
        String levelString = "";
        try {
            levelString = Decoder.decodeLevel(level.getEncodedLevelString()).replace(";", ";\n").replace("_", ",");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        area = new TextArea(levelString);
        pane.getChildren().add(area);

        Button save = new Button("Save");
        save.setDisable(true);
        Button export = new Button("Export");
        export.setOnMouseClicked(mouseEvent -> {
            export(pane, level, area.getText());
        });
        toolBar = new ToolBar(save, export);
        pane.getChildren().add(toolBar);

        return pane;
    }

    @Override
    public String getTitle() {
        return "Level string";
    }

    @Override
    public void setupResize(TabPane parent) {
        parent.widthProperty().addListener((observableValue, number, t1) -> {
            toolBar.setPrefWidth(t1.floatValue());
            area.setPrefWidth(t1.floatValue());
        });

        parent.heightProperty().addListener((observableValue, number, t1) -> {
            area.setPrefHeight(t1.floatValue() - toolBar.getHeight() - parent.getTabMaxHeight() - 7);
        });
    }

    public void export(Node node, Level level, String levelString) {
        FileChooser chooser = new FileChooser();
        chooser.setTitle("Save " + level.getName() + ".gmd");
        chooser.setInitialFileName(level.getName() + ".gmd");
        File file = chooser.showSaveDialog(node.getScene().getWindow());
        if (file != null) {
            try {
                UIProgressBar progressBar = new UIProgressBar();
                progressBar.openNow("Exporting level...");
                new Thread(() -> {
                    try {
                        Decoder.exportGMD(level.getName(), levelString, file.getPath(), progressBar, 1);
                        progressBar.close();
                    } catch (Exception e) {
                        throw new RuntimeException(e);
                    }
                }).start();
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }
}
